/*********************************************************************
 * TTCE 0.2.0: Taxes, Tips and Currencies in Euro
 * (C) Jrme Euzenat, 1998, 2000-2001
 *
 * Parts:
 * (C) Alan Harder, 1999
 * From TipMe v1.23 - 3/29/99
 * Alan.Harder@Sun.COM     http://www.moshpit.org/pilot/
 * Freeware - feel free to distribute and/or modify this code.. just
 *            leave this header on it.
 ********************************************************************/

/************************
 * IMPROVEMENT
 * 
 * It would be nice to simptoly put the field numbers as field handlers and let the 
 * system play ???
 */

/************************
 * CODING ISSUES
 * We want to optimize size, not speed
 *
 * From the given examples, we must determines if:
 * - It is better to compute things twice or to store them in variables (FP seem to think that variables are better)
 * - What is the price for a form
 * - Is it possible to share menus and so on...
 */

#include "ttce.h"
#include <GuiTools.h>
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#define CREATOR		'TTCE'
#define PREF_VERSION	1

#define TRUE 1
#define FALSE 0
#define ERROR -1
#define MAIN 0

// This takes into account default values
Char deciSep[] = ".", deciNoDime[] = ".0";
Char deciChr = '.';

Long total, amount, tip, tax, tax2, tipbase, ctotal;
FieldPtr amountFld, taxFld, taxAmtFld, tax2Fld, tax2AmtFld,
         tipFld, tipAmtFld, totalFld, currFld, currAmtFld;
Word currentField = NULL, lastField=NULL;

// App Preferences Type
typedef struct
{
    Int taxRate, tipRate, tax2Rate, currRate;
    Boolean useTax;
}
TtcePrefsType;

TtcePrefsType *prefs;

#define RETROCOMPALPHA() amount=(10000*total)/(10000+GetAmountFromField(taxFld)+GetAmountFromField(tax2Fld)+GetAmountFromField(tipFld));

static CharPtr currSymbols[5] = {"\200", "$", "", "", ""};

static int currIndex = 0;

// Methods to get and set Amounts in text fields.  An amount is the
// number of CENTS, and will be displayed with the decimal point placed
// appropriately..

void PutAmountInField(FieldPtr fieldPtr, Long amount)
{
    CharPtr str;
    Long amt;

    str = (CharPtr)MemPtrNew(16);
    if (amount < 0)
    {
	amt = -amount;
	StrCopy(str, "(");
    }
    else
    {
	amt = amount;
	StrCopy(str, "");
    }
    StrIToA(str + StrLen(str), amt/100);
    StrCat(str, (amt%100 < 10) ? deciNoDime : deciSep);
    StrIToA(str + StrLen(str), amt%100);
    if (amount < 0) StrCat(str, ")");
    SetText(fieldPtr, str);
    MemPtrFree(str);
}

Long GetAmountFromField(FieldPtr fieldPtr)
{
    Long val;
    Int cents;
    CharPtr str, str2;
    str = GetText(fieldPtr);
    if (str == NULL) return ERROR;
    if (StrLen(str) == 0) { MemPtrFree(str); return ERROR; }
    val = StrAToI(str) * 100;
    str2 = StrChr(str, deciChr);
    if (str2 != NULL && str2[1] >= '0' && str2[1] <= '9')
    {
	cents = (str2[1] - '0') * 10;
	if (str2[2] >= '0' && str2[2] <= '9')
	    cents += str2[2] - '0';
	val += cents;
    }
    MemPtrFree(str);
    return val;
}

// The default part of editors
Boolean EditFormEventHandler( FormPtr f, EventType *pEvent )
{
  switch (pEvent->eType)
  {
    case frmOpenEvent:
      f = FrmGetActiveForm();
      FrmDrawForm(f);
      // test if new or edit...
      FrmSetFocusById(f, nameEditField); //if new else, put on value
    return TRUE;

  case ctlSelectEvent:
    switch (pEvent->data.ctlSelect.controlID)
    {
    case cancelEditButton:
      FrmReturnToForm(editListForm);
      return TRUE;
    case resetEditButton:
      // should go in the not generic
      return TRUE;
    case saveEditButton:
      // should go in the not generic
      FrmReturnToForm(mainForm);
      return TRUE;
    }
  }
  return FALSE;
}

Boolean CurrFormEventHandler(EventType *pEvent)
{ FormPtr f;
    
  switch (pEvent->eType)
  {
    case ctlSelectEvent:
    if ( pEvent->data.ctlSelect.controlID == pivotCurrSymbolField ) {
      currIndex = (currIndex+1) % 5;
      CtlSetLabel( (ControlPtr)FrmGetObjectPtrById(FrmGetActiveForm(), pivotCurrSymbolField), currSymbols[currIndex]);
      return TRUE;
    }
  }
  return EditFormEventHandler( FrmGetActiveForm(), pEvent );
}

Boolean TaxFormEventHandler(EventType *pEvent)
{
    FormPtr f;
    
    // a large part of the handler will be shared by 3 editors
    // it can be put in a default handler
  switch (pEvent->eType)
  {
    case frmOpenEvent:
      f = FrmGetActiveForm();
      FrmDrawForm(f);
      // test if new or edit...
      FrmSetFocusById(f, nameEditField); //if new else, put on value
    return TRUE;

  case ctlSelectEvent:
    switch (pEvent->data.ctlSelect.controlID)
    {
    case cancelEditButton:
      FrmGotoForm(editListForm);
      return TRUE;
    case resetEditButton:
      return TRUE;
    case saveEditButton:
      FrmReturnToForm(mainForm);
      return TRUE;
      // There are also fields, checkBox and popUpLists...
      //-- For the currency we must have
    case pivotCurrSymbolField:
      currIndex = (currIndex+1) % 5;
      CtlSetLabel( (ControlPtr)FrmGetObjectPtrById(FrmGetActiveForm(), pivotCurrSymbolField), currSymbols[currIndex]);
      return TRUE;
    }
  }

  return FALSE;
}

void computeValues() {
	Long taxRate, tax2Rate, tipRate, currRate;
    amount = GetAmountFromField(amountFld);
    if ( (taxRate = GetAmountFromField(taxFld)) > 0) tax = (amount*taxRate+5000)/10000;	// standard rounding
    else  tax = 0;
    PutAmountInField(taxAmtFld, tax); //FldDrawField(taxAmtFld);
    if ( (tax2Rate = GetAmountFromField(tax2Fld)) > 0) tax2 = (amount*tax2Rate+5000)/10000;	// standard rounding
    else  tax2 = 0;
    PutAmountInField(tax2AmtFld, tax2);
    if ( (tipRate = GetAmountFromField(tipFld)) > 0) tip = (amount*tipRate+9999)/10000;	// always round up
    else  tip = 0;
    PutAmountInField(tipAmtFld, tip);
    total = amount + tax + tax2 + tip;
    PutAmountInField(totalFld, total);
    if ( (currRate = GetAmountFromField(currFld)) > 0) ctotal = total * currRate/100;
    PutAmountInField(currAmtFld, ctotal);
    //FrmDrawForm(FrmGetActiveForm());
    FrmUpdateForm( mainForm, frmRedrawUpdateCode );
}


Boolean MenuEventHandler(EventPtr event)
{
    switch ( event->data.menu.itemID ) {
      case menuItemAbout:
	FrmPopupForm(aboutForm) ;
        return TRUE;
      case menuItemEdit:
	FrmPopupForm(editListForm) ;
        return TRUE;
      case menuItemCut:
      case menuItemCopy:
      case menuItemPaste:
	{
	    FormPtr form ;
	    Word tmp_index ;
	    FieldPtr tmp_fld ;
	    
	    /* JE//: from FP, certainly size optimization that I do not know */
	    form = FrmGetActiveForm () ;
	    tmp_index = FrmGetFocus (form) ;
	    if (tmp_index != -1) {
		/* On croise les doigts pour qu'il n'y ait que les */
		/* fields qui puissent avoir le focus !!! */
		tmp_fld = FrmGetObjectPtr (form, tmp_index) ;
		switch (event->data.menu.itemID) {
		  case menuItemCut: FldCut (tmp_fld) ; break ;
		  case menuItemPaste: FldPaste (tmp_fld) ; break ;
		  case menuItemCopy: FldCopy (tmp_fld) ; break ;
		}
	    }
	    return TRUE;
	}
      default:
	FrmHandleEvent(FrmGetActiveForm(), event); /// This should be useless! Test?
        return TRUE;
        //return FALSE;
    };
    return FALSE;
}

void clearMainForm( FormPtr f ){
// These may be handlers? FltSetTextHandle( taxFld, (Handle)prefs->taxRate )
// sauf que taxRate est un entier et non une chaine
  LstSetSelection((ListPtr)FrmGetObjectPtrById( f, currList), 0);
  PutAmountInField(amountFld, 0);
  LstSetSelection((ListPtr)FrmGetObjectPtrById( f, taxList), 0);
  PutAmountInField(taxFld, 0);
  PutAmountInField(taxAmtFld, 0);
  LstSetSelection((ListPtr)FrmGetObjectPtrById( f, tax2List), 0);
  PutAmountInField(tax2Fld, 0);
  PutAmountInField(tax2AmtFld, 0);
  LstSetSelection((ListPtr)FrmGetObjectPtrById( f, tipList), 0);
  PutAmountInField(tipFld, 0);
  PutAmountInField(tipAmtFld, 0);
  PutAmountInField(totalFld, 0);
  LstSetSelection((ListPtr)FrmGetObjectPtrById( f, curr2List), 0);
  PutAmountInField(currFld, 100);
  PutAmountInField(currAmtFld, 0);
}

void restoreMainForm(){
  // This could also take into account the database prefs
  if (prefs->taxRate > 0) PutAmountInField(taxFld, prefs->taxRate);
  if (prefs->tax2Rate > 0) PutAmountInField(tax2Fld, prefs->tax2Rate);
  if (prefs->tipRate > 0) PutAmountInField(tipFld, prefs->tipRate);
  if (prefs->currRate > 0) PutAmountInField(currFld, prefs->currRate);
}

void updatePanel( Word fieldID )
{
	    switch ( fieldID ) {
	      case taxAmountField:
		tax = GetAmountFromField(taxAmtFld);
		if(amount>0)PutAmountInField(taxFld, tax*10000/amount);
		break;
	      case tax2AmountField:
		tax2 = GetAmountFromField(tax2AmtFld);
		//x/ if((amount+tax)>0)PutAmountInField(tax2Fld,tax2*10000/(amount+tax));
		if(amount>0)PutAmountInField(tax2Fld,tax2*10000/amount);
		break;
	      case tipAmountField:
		tip = GetAmountFromField(tipAmtFld);
		if(amount>0)PutAmountInField(tipFld,tip*10000/amount);
		break;
	      case totalField:
		total = GetAmountFromField(totalFld);
		RETROCOMPALPHA()
		PutAmountInField(amountFld,amount);
		break;
	      case curr2AmountField:
		ctotal = GetAmountFromField(currAmtFld);
		if(GetAmountFromField(currFld)>0)total=ctotal/GetAmountFromField(currFld)*100;
		PutAmountInField(totalFld,total);
		RETROCOMPALPHA()
		PutAmountInField(amountFld,amount);
		break;
	    }
	    computeValues();
}

Boolean MainFormEventHandler(EventPtr event)
{ FormPtr frm;

	switch (event->eType)
	{
	  case frmLoadEvent:
	    if (event->data.frmLoad.formID == mainForm) {
	    frm = FrmGetActiveForm();
	    /// Maybe this should not be put at loadTime but at creationTime. No?
	    amountFld = (FieldPtr)FrmGetObjectPtrById(frm, amountField);
	    taxFld = (FieldPtr)FrmGetObjectPtrById(frm, taxRateField);
	    taxAmtFld = (FieldPtr)FrmGetObjectPtrById(frm,taxAmountField);
	    tax2Fld = (FieldPtr)FrmGetObjectPtrById(frm, tax2RateField);
	    tax2AmtFld = (FieldPtr)FrmGetObjectPtrById(frm,tax2AmountField);
	    tipFld = (FieldPtr)FrmGetObjectPtrById(frm, tipRateField);
	    tipAmtFld = (FieldPtr)FrmGetObjectPtrById(frm,tipAmountField);
	    totalFld = (FieldPtr)FrmGetObjectPtrById(frm, totalField);
	    currFld = (FieldPtr)FrmGetObjectPtrById(frm,curr2RateField);
	    currAmtFld = (FieldPtr)FrmGetObjectPtrById(frm,curr2AmountField);
	    //--
	    clearMainForm(frm);
	    restoreMainForm();
	    }
            return TRUE;

	  case frmOpenEvent:
	    frm = FrmGetActiveForm();
	    FrmDrawForm(frm);
	    if (event->data.frmOpen.formID == mainForm) 
	    //{
		// Set the handlers for all the fields...
		// FldSetTextHandle(amountFld,(Handle)XX);
		// or Get the penUpEvent (as good)
		FrmSetFocusById( frm, ( GetAmountFromField(tipFld) > 0) ? amountField : tipRateField );
	    //};
            return TRUE;

	  case menuEvent:
	    if ( lastField ) updatePanel( lastField );
	    currentField = lastField = NULL;
	    MenuEventHandler(event);
            return TRUE;

	  case penDownEvent:
	    lastField = currentField;
	    currentField = NULL;
	    return FALSE;

	  case fldEnterEvent:
	    currentField = event->data.fldEnter.fieldID;
	    FrmHandleEvent(FrmGetActiveForm(), event); // default action
            return TRUE;
	    
	  //case keyDownEvent: // keyDownEvent, instead of penUpEvent, taken from currex!
	    // but keyDown is it sent when a button or a trigger are selected?
	    //if ( ( 0x30 <= event->date.keyDown.chr <= 0x39 ) )
	  case penUpEvent:
	    //if ( lastField != currentField ) updatePanel( lastField );
	    if (lastField && (event->screenY > 20)) updatePanel( lastField ); // Ugly
	      FrmHandleEvent(FrmGetActiveForm(), event); // default action
            return TRUE;
	    
	  // We must do with the Triggers
	  case popSelectEvent:
	    //currentField = event->data.popSelect.listID;
	    switch ( currentField ) {
	      case ttceCategoryTrigger:
		// reset the associated from the database
		break;
	      case taxList:
		// Must be replaced by database lookup
		switch (event->data.popSelect.selection) {
		    case 1: PutAmountInField(taxFld, 1960);break ;
		    case 2: PutAmountInField(taxFld, 450);break ;
		    default: PutAmountInField(taxFld, 0);
		}
	        computeValues();
                break ;
	    };
	    FrmHandleEvent(FrmGetActiveForm(), event); // default action
            return TRUE;

	  case ctlSelectEvent:
	    switch (event->data.ctlSelect.controlID)
	    {
	      case calcBtn:
		computeValues();
                return TRUE;

	      case compRateBtn:
		// never used
		if(total>0)PutAmountInField(currFld,ctotal*100/total);
                return TRUE;

	      case recBtn:
		prefs->taxRate = GetAmountFromField(taxFld);
		prefs->tax2Rate = GetAmountFromField(tax2Fld);
		prefs->tipRate = GetAmountFromField(tipFld);
		prefs->currRate = GetAmountFromField(currFld);
                return TRUE;

	      case resBtn:
	        restoreMainForm();
		FrmSetFocusById( FrmGetActiveForm(), amountField);
		computeValues();
                return TRUE;

	      case clearBtn:
		frm = FrmGetActiveForm();
	        clearMainForm(frm);
		FrmSetFocusById( frm, amountField);
		FrmDrawForm(frm);
                return TRUE;

	      case okAboutBtn:
		FrmReturnToForm(mainForm);
                return TRUE;
	
	      case delSelectButton:
		// Delete the selected item
                return TRUE;
		
	      case newSelectButton:
		// Just bring the new form
                FrmGotoForm( taxEditForm + LstGetSelection((ListPtr)FrmGetObjectPtrById(FrmGetActiveForm(), ttceCategoryList)) );//taxEditForm==5100
                return TRUE;
		
	      case modSelectButton:
		// Set a variable on the selected stuff and do no do anything if it does not exists.
		FrmGotoForm( taxEditForm + LstGetSelection((ListPtr)FrmGetObjectPtrById(FrmGetActiveForm(), ttceCategoryList)) );//taxEditForm==5100
                return TRUE;
		
	      case okSelectButton:
		FrmReturnToForm(mainForm);
                return TRUE;
	    }

	  default:
	    FrmHandleEvent(FrmGetActiveForm(), event);
            return TRUE;
	}
	return FALSE;
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    EventType evt;
    Word error, prefsSize;
    FormPtr frm;
    Long intVal;
    Int type;

    if (cmd != sysAppLaunchCmdNormalLaunch) return FALSE;

    type = PrefGetPreference(prefNumberFormat);
    if (type==nfPeriodComma || type==nfSpaceComma || type==nfApostropheComma)
    {
	deciChr = deciSep[0] = deciNoDime[0] = ',';
    }

    FrmGotoForm(mainForm);

    prefsSize = sizeof(TtcePrefsType);
    prefs = MemPtrNew(sizeof(TtcePrefsType));
    if (PrefGetAppPreferences(CREATOR, PREF_VERSION,
			      prefs, &prefsSize, true) == noPreferenceFound)
    {
	prefs->tipRate = prefs->taxRate = prefs->tax2Rate = 0;
	prefs->currRate = 100;
    }

    do {
	EvtGetEvent(&evt, evtWaitForever);
	if (SysHandleEvent(&evt)) continue;
	if (MenuHandleEvent(NULL, &evt, &error)) continue;
	
        if (evt.eType == frmLoadEvent)
        {
                Word Id = evt.data.frmLoad.formID;
		frm = FrmInitForm(Id) ;
		FrmSetActiveForm(frm) ;
		switch (Id) {
		  case taxEditForm:
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)TaxFormEventHandler) ;
		    break;
		  case tipEditForm:
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)TaxFormEventHandler) ;
		    break;
		  case currEditForm:
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)TaxFormEventHandler) ; //CurrFormEventHandler
		    break;
		  default: //  mainForm aboutForm editListForm
		    FrmSetEventHandler(frm, (FormEventHandlerPtr)MainFormEventHandler) ;
		}
        } //else FrmHandleEvent(FrmGetActiveForm(), &evt);
	FrmDispatchEvent(&evt);
    } while ( evt.eType != appStopEvent );
    PrefSetAppPreferences(CREATOR, PREF_VERSION, PREF_VERSION,
				      prefs, sizeof(TtcePrefsType), true);
    MemPtrFree(prefs);
    // Unset the handlers for all the fields...
    FldSetTextHandle(amountFld,NULL);
    FldSetTextHandle(tipFld,NULL);
    FldSetTextHandle(taxFld,NULL);
    FldSetTextHandle(tax2Fld,NULL);
    FldSetTextHandle(totalFld,NULL);
    FldSetTextHandle(currFld,NULL);
    FrmCloseAllForms();
    return FALSE;
}
